@@ -1,7 +1,6 @@
-/* $Id: gucharmap-block-chapters.c 1376 2007-12-03 21:42:48Z chpe $ */
 /*
- * Copyright (c) 2004 Noah Levitt
- * Copyright (C) 2007 Christian Persch
+ * Copyright © 2004 Noah Levitt
+ * Copyright © 2007 Christian Persch
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -15,13 +14,15 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
  */
 
-#include "config.h"
-#include "gucharmap-block-chapters-model.h"
-#include "gucharmap-intl.h"
-#include "gucharmap-settings.h"
+#include <config.h>
+#include <glib/gi18n-lib.h>
+
+#include "gucharmap.h"
+#include "gucharmap-private.h"
+
 #include "unicode-blocks.h"
 
 enum 
@@ -55,10 +56,14 @@ gucharmap_block_chapters_model_init (GucharmapBlockChaptersModel *model)
 
   for (i = 0;  i < G_N_ELEMENTS (unicode_blocks); i++)
     {
+      const char *block_name;
+
+      block_name = unicode_blocks_strings + unicode_blocks[i].block_name_index;
+
       gtk_list_store_append (store, &iter);
       gtk_list_store_set (store, &iter,
-                          BLOCK_CHAPTERS_MODEL_ID, unicode_blocks[i].block_name,
-                          BLOCK_CHAPTERS_MODEL_LABEL, _(unicode_blocks[i].block_name), 
+                          BLOCK_CHAPTERS_MODEL_ID, block_name,
+                          BLOCK_CHAPTERS_MODEL_LABEL, _(block_name),
                           BLOCK_CHAPTERS_MODEL_UNICODE_BLOCK_PTR, unicode_blocks + i,
                           -1);
     }
@@ -75,18 +80,21 @@ get_codepoint_list (GucharmapChaptersModel *chapters,
 
   /* special "All" block */
   if (unicode_block == NULL)
-    return gucharmap_codepoint_list_new (0, UNICHAR_MAX);
+    return gucharmap_block_codepoint_list_new (0, UNICHAR_MAX);
     
-  return gucharmap_codepoint_list_new (unicode_block->start, unicode_block->end);
+  return gucharmap_block_codepoint_list_new (unicode_block->start, unicode_block->end);
 }
 
-static G_CONST_RETURN GucharmapCodepointList * 
-get_book_codepoint_list (GucharmapChaptersModel *chapters)
+static GucharmapCodepointList *
+get_book_codepoint_list (GucharmapChaptersModel *model)
 {
-  if (chapters->book_list == NULL)
-    chapters->book_list = gucharmap_codepoint_list_new (0, UNICHAR_MAX);
+  GucharmapChaptersModelPrivate *model_priv = model->priv;
 
-  return chapters->book_list;
+  if (model_priv->book_list == NULL) {
+    model_priv->book_list = gucharmap_block_codepoint_list_new (0, UNICHAR_MAX);
+  }
+
+  return g_object_ref (model_priv->book_list);
 }
 
 /* XXX linear search */
