@@ -1,6 +1,5 @@
-/* $Id: gucharmap-unicode-info.c 1173 2006-07-24 23:06:31Z behdad $ */
 /*
- * Copyright (c) 2004 Noah Levitt
+ * Copyright © 2004 Noah Levitt
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -14,29 +13,28 @@
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * 59 Temple Place, Suite 330, Boston, MA 02110-1301  USA
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <gtk/gtk.h>
 #include <string.h>
-#include "gucharmap-intl.h"
-#include "gucharmap-unicode-info.h"
+
+#include "gucharmap.h"
+#include "gucharmap-private.h"
 
 #include "unicode-names.h"
 #include "unicode-blocks.h"
 #include "unicode-nameslist.h"
 #include "unicode-categories.h"
+#include "unicode-versions.h"
 #if ENABLE_UNIHAN
 # include "unicode-unihan.h"
 #endif
 
 /* constants for hangul (de)composition, see UAX #15 */
 #define SBase 0xAC00
-#define LBase 0x1100
-#define VBase 0x1161
-#define TBase 0x11A7
 #define LCount 19
 #define VCount 21
 #define TCount 28
@@ -67,7 +65,7 @@ gucharmap_get_unicode_name (gunichar wc)
 
   _gucharmap_intl_ensure_initialized ();
 
-  if ((wc >= 0x3400 && wc <= 0x4DB5) 
+  if ((wc >= 0x3400 && wc <= 0x4DB5)
       || (wc >= 0x4e00 && wc <= 0x9fa5) 
       || (wc >= 0x20000 && wc <= 0x2A6D6))
     {
@@ -155,65 +153,6 @@ gucharmap_get_unicode_category_name (gunichar wc)
     }
 }
 
-/* http://www.unicode.org/unicode/reports/tr15/#Hangul */
-static gunichar *
-hangul_decomposition (gunichar s, gsize *result_len)
-{
-  gunichar *r = g_malloc (3 * sizeof (gunichar));
-  gint SIndex = s - SBase;
-
-  /* not a hangul syllable */
-  if (SIndex < 0 || SIndex >= SCount)
-    {
-      r[0] = s;
-      *result_len = 1;
-    }
-  else
-    {
-      gunichar L = LBase + SIndex / NCount;
-      gunichar V = VBase + (SIndex % NCount) / TCount;
-      gunichar T = TBase + SIndex % TCount;
-
-      r[0] = L;
-      r[1] = V;
-
-      if (T != TBase) 
-        {
-          r[2] = T;
-          *result_len = 3;
-        }
-      else
-        *result_len = 2;
-    }
-
-  return r;
-}
-
-/*
- * See http://bugzilla.gnome.org/show_bug.cgi?id=100456
- *
- * gucharmap_unicode_canonical_decomposition:
- * @ch: a Unicode character.
- * @result_len: location to store the length of the return value.
- *
- * Computes the canonical decomposition of a Unicode character.  
- * 
- * Return value: a newly allocated string of Unicode characters.
- *   @result_len is set to the resulting length of the string.
- */
-gunichar *
-gucharmap_unicode_canonical_decomposition (gunichar ch, 
-                                           gsize *result_len)
-{
-  if (GLIB_CHECK_VERSION (2,3,1))
-    return g_unicode_canonical_decomposition (ch, result_len);
-  else
-    if (ch >= 0xac00 && ch <= 0xd7af)  /* Hangul syllable */
-      return hangul_decomposition (ch, result_len);
-    else 
-      return g_unicode_canonical_decomposition (ch, result_len);
-}
-
 /* does a binary search on unicode_names */
 G_CONST_RETURN gchar *
 gucharmap_get_unicode_data_name (gunichar uc)
@@ -245,6 +184,44 @@ gucharmap_get_unicode_data_name_count (void)
   return G_N_ELEMENTS (unicode_names);
 }
 
+/* does a binary search on unicode_versions */
+GucharmapUnicodeVersion
+gucharmap_get_unicode_version (gunichar uc)
+{
+  gint min = 0;
+  gint mid;
+  gint max = G_N_ELEMENTS (unicode_versions) - 1;
+
+  if (uc < unicode_versions[0].start || uc > unicode_versions[max].end)
+    return GUCHARMAP_UNICODE_VERSION_UNASSIGNED;
+
+  while (max >= min)
+    {
+      mid = (min + max) / 2;
+
+      if (uc > unicode_versions[mid].end)
+        min = mid + 1;
+      else if (uc < unicode_versions[mid].start)
+        max = mid - 1;
+      else if ((uc >= unicode_versions[mid].start) && (uc <= unicode_versions[mid].end))
+        return unicode_versions[mid].version;
+    }
+
+  return GUCHARMAP_UNICODE_VERSION_UNASSIGNED;
+}
+
+G_CONST_RETURN gchar *
+gucharmap_unicode_version_to_string (GucharmapUnicodeVersion version)
+{
+  g_return_val_if_fail (version >= GUCHARMAP_UNICODE_VERSION_UNASSIGNED &&
+                        version <= GUCHARMAP_UNICODE_VERSION_LATEST, NULL);
+
+  if (G_UNLIKELY (version == GUCHARMAP_UNICODE_VERSION_UNASSIGNED))
+    return NULL;
+
+  return unicode_version_strings + unicode_version_string_offsets[version - 1];
+}
+
 #if ENABLE_UNIHAN
 
 gint
@@ -352,7 +329,7 @@ get_nameslist (gunichar uc)
   static const NamesList *most_recent_result;
   gint min = 0;
   gint mid;
-  gint max = sizeof (names_list) / sizeof (NamesList) - 1;
+  gint max = G_N_ELEMENTS (names_list) - 1;
 
   if (uc < names_list[0].index || uc > names_list[max].index)
     return NULL;
@@ -380,9 +357,7 @@ get_nameslist (gunichar uc)
   return NULL;
 }
 
-gboolean _gucharmap_unicode_has_nameslist_entry (gunichar uc);
-
-gboolean
+G_GNUC_INTERNAL gboolean
 _gucharmap_unicode_has_nameslist_entry (gunichar uc)
 {
   return get_nameslist (uc) != NULL;
@@ -394,7 +369,7 @@ gucharmap_get_nameslist_exes (gunichar uc)
 {
   const NamesList *nl;
   gunichar *exes;
-  gint i, count;
+  gunichar i, count;
   
   nl = get_nameslist (uc);
 
@@ -420,8 +395,8 @@ gucharmap_get_nameslist_equals (gunichar uc)
 {
   const NamesList *nl;
   const gchar **equals;
-  gint i, count;
-  
+  gunichar i, count;
+
   nl = get_nameslist (uc);
 
   if (nl == NULL || nl->equals_index == -1)
@@ -433,7 +408,7 @@ gucharmap_get_nameslist_equals (gunichar uc)
 
   equals = g_malloc ((count + 1) * sizeof (gchar *));
   for (i = 0;  i < count;  i++)
-    equals[i] = names_list_equals[nl->equals_index + i].value;
+    equals[i] = names_list_equals_strings + names_list_equals[nl->equals_index + i].string_index;
   equals[count] = NULL;
 
   return equals;
@@ -446,7 +421,7 @@ gucharmap_get_nameslist_stars (gunichar uc)
 {
   const NamesList *nl;
   const gchar **stars;
-  gint i, count;
+  gunichar i, count;
 
   nl = get_nameslist (uc);
 
@@ -459,7 +434,7 @@ gucharmap_get_nameslist_stars (gunichar uc)
 
   stars = g_malloc ((count + 1) * sizeof (gchar *));
   for (i = 0;  i < count;  i++)
-    stars[i] = names_list_stars[nl->stars_index + i].value;
+    stars[i] = names_list_stars_strings + names_list_stars[nl->stars_index + i].string_index;
   stars[count] = NULL;
 
   return stars;
@@ -472,7 +447,7 @@ gucharmap_get_nameslist_pounds (gunichar uc)
 {
   const NamesList *nl;
   const gchar **pounds;
-  gint i, count;
+  gunichar i, count;
   
   nl = get_nameslist (uc);
 
@@ -485,7 +460,7 @@ gucharmap_get_nameslist_pounds (gunichar uc)
 
   pounds = g_malloc ((count + 1) * sizeof (gchar *));
   for (i = 0;  i < count;  i++)
-    pounds[i] = names_list_pounds[nl->pounds_index + i].value;
+    pounds[i] = names_list_pounds_strings + names_list_pounds[nl->pounds_index + i].string_index;
   pounds[count] = NULL;
 
   return pounds;
@@ -498,8 +473,8 @@ gucharmap_get_nameslist_colons (gunichar uc)
 {
   const NamesList *nl;
   const gchar **colons;
-  gint i, count;
-  
+  gunichar i, count;
+
   nl = get_nameslist (uc);
 
   if (nl == NULL || nl->colons_index == -1)
@@ -511,23 +486,17 @@ gucharmap_get_nameslist_colons (gunichar uc)
 
   colons = g_malloc ((count + 1) * sizeof (gchar *));
   for (i = 0;  i < count;  i++)
-    colons[i] = names_list_colons[nl->colons_index + i].value;
+    colons[i] = names_list_colons_strings + names_list_colons[nl->colons_index + i].string_index;
   colons[count] = NULL;
 
   return colons;
 }
 
-#define UNICODE_VALID(Char)                   \
-    ((Char) < 0x110000 &&                     \
-    ((Char) < 0xD800 || (Char) >= 0xE000) &&  \
-    ((Char) < 0xFDD0 || (Char) > 0xFDEF) &&   \
-    ((Char) & 0xFFFF) != 0xFFFE && ((Char) & 0xFFFF) != 0xFFFF)
-
-/* an up-to-date replacement for g_unichar_validate */
+/* Wrapper, in case we want to support a newer unicode version than glib */
 gboolean
 gucharmap_unichar_validate (gunichar ch)
 {
-  return UNICODE_VALID (ch);
+  return g_unichar_validate (ch);
 }
 
 /**
@@ -655,3 +624,40 @@ gucharmap_unichar_isgraph (gunichar uc)
           && t != G_UNICODE_SPACE_SEPARATOR);
 }
 
+static gunichar
+get_first_non_underscore_char (const char *str)
+{
+  const char *p;
+
+  if (!str)
+    return 0;
+
+  for (p = str; p && *p; p = g_utf8_find_next_char (p, NULL))
+    {
+      gunichar ch;
+
+      ch = g_utf8_get_char (p);
+      if (g_unichar_isalpha (ch))
+        return ch;
+    }
+
+  return 0;
+}
+
+/**
+ * gucharmap_unicode_get_locale_character:
+ *
+ * Determines a character that's commonly used in the current
+ * locale's script.
+ * 
+ * Returns: a unicode character
+ */
+gunichar
+gucharmap_unicode_get_locale_character (void)
+{
+  GtkStockItem item;
+  if (!gtk_stock_lookup (GTK_STOCK_FIND, &item))
+    return 0;
+
+  return get_first_non_underscore_char (item.label);
+}
